from urllib.parse import urlencode
from urllib.request import Request, urlopen
from json import loads
from os.path import join, dirname
import pytest


class TestSimulation(object):
    """
    A 'testing' class used to group all the tests related to the simulation data
    """

    def test_simulation(self, queries_to_test):
        """
        This is the entry point of the test. This method will be found and executed when the module is ran using pytest

        :param queries_to_test: the fixture to use - returns a JSON object represented as a python dictionary
        """

        print("\n")  # prints a blank line for formatting purposes

        # go through all query references in the dictionary and execute a test for each of them
        for query_key in queries_to_test:
            query_to_execute = queries_to_test[query_key]["query"]
            query_actual_result = self.send_query("http://localhost:8086", query_to_execute)
            query_expected_result = queries_to_test[query_key]["expected_result"]

            self.check_result(query_key, query_actual_result, query_expected_result, queries_to_test.keys())
            print("Successfully passed test for '{0}' query".format(query_key))

    @staticmethod
    def check_result(query_key, query_actual_result, query_expected_result, queries_under_test):
        """
        A test for a single query result given the query reference and the actual result from executing it

        :param query_key: the executed query reference, which is under test
        :param query_actual_result: the result from the executed query
        :param query_expected_result: the expected result for the executed query
        :param queries_under_test: the set of query keys, which are being tested
        """

        assert query_key is not None, "The query key argument must be an existing object."
        assert query_actual_result is not None, "The query's actual result argument must be an existing object."

        assert query_key in queries_under_test, "The query reference {0} is not found in the queries under test.".format(query_key)

        assert query_expected_result == query_actual_result, "Simulation test failure"

    @staticmethod
    def send_query(url, query):
        """
        An auxiliary static method to send a query to a url and retrieve the result

        :param url: the target url to which the query is sent to - a string containing a valid URL address
        :param query: the query to be executed on the given URL
        :return: the result of the executed query
        """

        query = urlencode({"q": query}).encode("ascii")
        request = Request("{0}/query".format(url), query)
        result = urlopen(request)

        return result.read().decode("utf-8").strip()

    @staticmethod
    @pytest.fixture
    def queries_to_test():
        """
        A pytest fixture used to read the queries, which would be tested, from a JSON file
        :return: a JSON object represented as a python dictionary
        """

        to_read = join(dirname(__file__), "queryTests.json")
        with open(to_read) as f:
            return loads(f.read().replace('\n', '').replace("'", "\\\""))
